#!/usr/bin/python2.2

## soundcardBackend.py - Contains the backend code needed for redhat-config-soundcard
## Copyright (C) 2002 Red Hat, Inc.
## Copyright (C) 2002 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import string
import kudzu
import os

class soundcardBackend:
    def __init__(self):
        self.defaultModule = None
        self.newFile = []
        self.cardList = []
        self.dspList = []
        self.dspListAlsa = []
	self.alsaList = []
	self.useAlsa = []
        self.readAlsa()
	# build a list of cards
	self.probeCards()
	# now check current configuration
        self.readFile(1)

    def readAlsa(self):
        self.alsaList = []
        fd = open('/usr/share/redhat-config-soundcard/alsa-pci', 'r')
        lines = fd.readlines()
        fd.close()
        for dev in lines:
	    if string.strip(dev)[0] != "#":
		vendorId, deviceId, module, description = string.split(dev, "\t")
    		maker, model = string.split(description, "|")
		self.alsaList.append((int(vendorId, 0x10), int(deviceId, 0x10), string.replace(module, "\"", "")))

    def findAlsa(self, vendorId, deviceId):
        for card in self.alsaList:
	    cardVendorId, cardDeviceId, module = card
	    if cardVendorId == vendorId and cardDeviceId == deviceId:
		return module
	return None
    
    def probeCards(self):
        self.cardList = []
        list = kudzu.probe(kudzu.CLASS_AUDIO, kudzu.BUS_PCI, kudzu.PROBE_ALL)

        for card in list:
            #Let's go through the list of cards.  If the driver is 'unknown',
            #don't put the card in the list.
            #if card.driver != "unknown":
	    device, module, description = card
    	    maker, model = string.split(description, "|")
	    alsaModule = self.findAlsa(card.vendorId, card.deviceId)
	    useAlsa = 0
	    if card.driver == "unknown":
		useAlsa = 1
	    cardA = (device, module, maker, model, alsaModule, useAlsa)
	    if card.driver != "unknown" or alsaModule != None:
        	self.cardList.append(cardA)
		if useAlsa:
		    self.useAlsa.append(alsaModule)

        return self.cardList

    def getData(self, card):
        #device, module, description = card
        #maker, model = string.split(description, "|")
	# find alsa driver for this card:
	#alsaModule = self.findAlsa(card.vendorId, card.deviceId)
	device, module, maker, model, alsaModule, useAlsa = card
	if self.useAlsa.count(alsaModule) > 0:
	    useAlsa = 1
	else:
	    useAlsa = 0
        return device, module, maker, model, alsaModule, useAlsa

    # inModule as really a way to select card, so old module may be passed to set this cart to use ALSA
    def setUseAlsaModule(self, inModule, value):
	if self.useAlsa.count(inModule) > 0 and value == 0:
	    self.useAlsa.remove(inModule)
	if self.useAlsa.count(inModule) == 0 and value == 1:
	    self.useAlsa.append(inModule)
	
    def insmod(self, module):
        pass

    def rmmod(self, module):
        pass

    def lsmod(self):
        pass

    def amixerSet(self, device, option, value):
        path = "/usr/bin/amixer"
        pid = os.fork()
        args = [path, "set", option, value, "unmute", "-q" ]

        if (not pid):
            os.execv(path, args)
	
    def playTestSoundKernel(self, module):
	if self.dspList.index(module) == 0:
    	    device = "/dev/dsp"
    	else:
    	    device = "/dev/dsp%d" % self.dspList.index(module)

    	path = "/usr/bin/play"
	pid = os.fork()
    	args = [path,  "-d", device, "/usr/share/redhat-config-soundcard/redhat-sample.wav", "-v", "2"]
	    
	if (not pid):
    	    os.execv(path, args)

    def playTestSoundAlsa(self, module):
    	if self.dspList.index(module) == 0:
    	    device = "hw:0,0"
    	else:
    	    device = "hw:%d,0" % self.dspList.index(module)
	    
	self.amixerSet(device, 'Master', '50%')
	self.amixerSet(device, 'Headphone', '50%')
	self.amixerSet(device, 'PCM', '50%')
	self.amixerSet(device, 'CD', '50%')
	
    	path = "/usr/bin/aplay"
	pid = os.fork()
    	args = [path, "-q", "-c", "2", "-t", "wav", "-D", device, "/usr/share/redhat-config-soundcard/redhat-sample.wav"]
    	#path = "/usr/bin/play"
	#pid = os.fork()
    	#args = [path,  "-d", device, "/usr/share/redhat-config-soundcard/redhat-sample.wav", "-v", "2"]
	    
	if (not pid):
    	    os.execv(path, args)

    def playTestSound(self, module, useAlsa):
	if useAlsa:
	    self.playTestSoundAlsa(module)
	else:
	    self.playTestSoundKernel(module)
            
    def readFile(self, storeUseAlsa):
	#self.prefAlsa = 0
        fd = open('/etc/modules.conf', 'r')
        lines = fd.readlines()
        fd.close()

        self.newFile = []

        for line in lines:
            #strip out all lines dealing with sound
            if (string.find(line, "alias") < 0) or (string.find(line, "sound-slot") < 0 and string.find(line, "snd-card") < 0  and string.find(line, "sound-service") < 0 and string.find(line, "alias char-major-116")) or string.strip(line)[0] == "#":
		if string.find(line, "alias") > -1 and string.find(line, "char-major-116") > -1:
		    continue
		if string.find(line, "alias") > -1 and string.find(line, "char-major-14") > -1:
		    continue
		# skip alsa mixer
		if string.find(line, "snd-card") > -1 and string.find(line, "alsactl") > -1:
		    continue
		# skip aumix mixer
		if string.find(line, "sound-slot") > -1 and string.find(line, "aumix") > -1:
		    continue
		self.newFile.append(line)

            else:
                if string.find(line, "sound-slot-0") > -1:
		    bakDefaultModule = self.defaultModule
                    try:
		        a, b, self.defaultModule = string.split(line)
			if self.defaultModule == "snd-card-0":
			    #if self.prefAlsa == 0:
				#self.prefAlsa = 1
				# not really module: restore previous info
                    		self.defaultModule = bakDefaultModule
                    		#self.defaultModule = None
				# and seek real module
			    #else:
				#self.defaultModule = bakDefaultModule
				#if a == "alias" and b == "sound-slot-0":
			#else:
                    	    #self.defaultOldModule = self.defaultModule
                    except:
                        self.defaultModule = None
                else:
		    if string.find(line, "snd-card-0") > -1:
                	try:
                    	    a, b, self.defaultModule = string.split(line)
			    if b == "snd-card-0":
				# use ALSA for this module
				if storeUseAlsa:
				    self.setUseAlsaModule(self.defaultModule, 1)
                	except:
                    	    self.defaultModule = None
	
    def getDefaultCard(self):
        for card in self.cardList:
            device, module, maker, model, moduleAlsa, useAlsa = self.getData(card)
            
            if module == self.defaultModule:
                return card
            if moduleAlsa == self.defaultModule:
                return card

    def writeFile(self, selectedCard):
        self.selectedCard = selectedCard
        aliasList = []
        mixerList = []
        self.dspList = []
        
        device, module, maker, model, alsaModule, useAlsa = self.getData(self.cardList[selectedCard])
	if useAlsa:
	    self.defaultModule = alsaModule
	else:
	    self.defaultModule = module
        aliasList.append("alias sound-slot-0 %s\n" % self.defaultModule)
	if useAlsa:
    	    mixerList.append("post-install snd-card-0 /usr/sbin/alsactl restore >/dev/null 2>&1 || :\n")
    	    mixerList.append("pre-remove snd-card-0 /usr/sbin/alsactl store >/dev/null 2>&1 || :\n")
	else:
    	    mixerList.append("post-install sound-slot-0 /bin/aumix-minimal -f /etc/.aumixrc -L >/dev/null 2>&1 || :\n")
    	    mixerList.append("pre-remove sound-slot-0 /bin/aumix-minimal -f /etc/.aumixrc -S >/dev/null 2>&1 || :\n")
	if useAlsa:
    	    aliasList.append("alias char-major-116 snd\n")
    	    #now e.g.:alias snd-card-0 snd-emu10k1
    	    aliasList.append("alias snd-card-0 %s\n" % self.defaultModule)
	    # OSS:
	    aliasList.append("alias char-major-14 soundcore\n")
	    # for all cards:
	    aliasList.append("alias sound-slot-0 snd-card-0\n")
	    aliasList.append("alias sound-service-0-0 snd-mixer-oss\n")
	    aliasList.append("alias sound-service-0-1 snd-seq-oss\n")
	    aliasList.append("alias sound-service-0-3 snd-pcm-oss\n")
	    aliasList.append("alias sound-service-0-8 snd-seq-oss\n")
	    aliasList.append("alias sound-service-0-12 snd-pcm-oss\n")
	
        self.dspList.append(self.defaultModule)

        count = 1
        for card in self.cardList:
            device, module, maker, model, alsaModule, useAlsa = self.getData(card)
	    currModule = module
	    if useAlsa:
		currModule = alsaModule

            if currModule == self.defaultModule:
                #We've already written this line, so skip
                pass
            else:
                aliasList.append("alias sound-slot-%d %s\n" % (count, currModule))
		if useAlsa:
    		    mixerList.append("post-install snd-card-%d /usr/sbin/alsactl restore >/dev/null 2>&1 || :\n" % count)
    		    mixerList.append("pre-remove snd-card-%d /usr/sbin/alsactl store >/dev/null 2>&1 || :\n" % count)
		else:
            	    mixerList.append("post-install sound-slot-%d /bin/aumix-minimal -f /etc/.aumixrc -L >/dev/null 2>&1 || :\n" % count)
            	    mixerList.append("pre-remove sound-slot-%d /bin/aumix-minimal -f /etc/.aumixrc -S >/dev/null 2>&1 || :\n" % count)
		if useAlsa:
    		    aliasList.append("alias snd-card-%d %s\n" % (count, currModule))
		    aliasList.append("alias sound-slot-%d snd-card-%d\n" % (count,count))
    		    aliasList.append("alias sound-service-%d-0 snd-mixer-oss\n" % count)
		    aliasList.append("alias sound-service-%d-3 snd-pcm-oss\n" % count)
		    aliasList.append("alias sound-service-%d-12 snd-pcm-oss\n" % count)
                self.dspList.append(currModule)
                count = count + 1

        #Let's re-read the file again to make sure we're starting from a clean base
	# but save alsa prefs
	#bakAlsa = self.prefAlsa
        self.readFile(0)
	
    	self.newFile = self.newFile + aliasList + mixerList

        fd = open('/etc/modules.conf', 'w')

        for line in self.newFile:
            fd.write(line)

        fd.close()

    def unloadModules(self):
        for card in self.cardList:
            device, module, maker, model, alsaModule, useAlsa = self.getData(card)
	
            retval = os.system("/sbin/modprobe -r %s" % module)
	    retval = os.system("/sbin/modprobe -r snd-pcm-oss")
	    retval = os.system("/sbin/modprobe -r snd-mixer-oss")
	    retval = os.system("/sbin/modprobe -r snd-seq-oss")
            retval = os.system("/sbin/modprobe -r %s" % alsaModule)

    def loadModules(self):
        retval = os.system("/sbin/modprobe %s" % self.defaultModule)
        if retval == 0:
            #Modprobe worked
            pass
        else:
            #Modprobe failed
            return self.defaultModule

    	#retval = os.system("/sbin/modprobe %s" % "snd-pcm-oss")
	#retval = os.system("/sbin/modprobe %s" % "snd-mixer-oss")
    	#retval = os.system("/sbin/modprobe %s" % "snd-seq-oss")
	
        for card in self.cardList:
            device, module, maker, model, alsaModule, useAlsa = self.getData(card)
	    if useAlsa:
		module = alsaModule

            if module == self.defaultModule:
                #We've already inserted this one, so don't try again
                pass
            else:
                retval = os.system("/sbin/modprobe %s" % module)

                if retval == 0:
                    #Modprobe worked
                    pass
                else:
                    #Modprobe failed
                    return self.defaultModule

        return 0

